/* Notes
	A plug in must not call a function, even a built in function. This will then make
	it relocatable and can be used in any ByPic project at any position. If a function
	is called this not only increases the complexity but will make it very specific to
	a device and release.
	A plug in requires a data location to be passed and will always return an integer.
	
	When the plugin is built, the .elf file along with pic32-objdump is used to produce a 
	disassembly of the code, this is placed in 'dis.tst'. The opcodes are taken from that 
	file are put in a constant {} data statement ready for the ByPic to use.
	
	This could be done manually but there is a A Python program called distobasX.py (X is a number) 
	that does this automatically. It will create 'dis.txt', parse it and output a file called 'plugin.bas'
	
	Compiler options: -mlong-calls -fPIC -mno-gpopt -mshared -mno-abicalls
*/
#define nop()  __asm__ __volatile__ ("%(ssnop%)" : :)
// Integer pointer arithmatic, 1 is 4 bytes
#define CLR  1
#define SET  2
#define INV  3

#define STANDARD // speed

#ifdef STANDARD
// compensate for code, know 1 uS off timings, i.e 5 
#define	A  25
#define	B  315
#define	C  295
#define	D  45
#define	E  40
#define	F  270
#define	G  1
#define	H  2395
#define	I  345
#define	J  2045
#endif

// 1 Wire
// Prep timer used needs to be set up to give 0,2 uS delay. The port needs weak pull ups
// enabled if this is not on the hardware
// Input locations
// 0 Timer register address, this will be for example TMR2
// 1 Timer pre register address example PMR2
// 2 IFSO address for timer flag
// 3 IFSO mask for required bit
// 4 TRIS address for port used
// 5 PORT addresss
// 6 LAT address
// 7 port mask for required bit
// 10 Byte to write	
int owRead_pi(char *c)
{
int *ip, *tmr, *pmr, *ifso, ifsomask, *tris, *port, *lat, mask;
int loop, inval, result=0;
	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	tmr = (int*)*(ip);
	pmr = (int*)*(ip+1);
	ifso = (int*)*(ip+2);
	ifsomask = *(ip+3);
	tris = (int*)*(ip+4);
	port = (int*)*(ip+5);
	lat = (int*)*(ip+6);
	mask = *(ip+7);

	for(loop=0; loop < 8; loop++) {
		result >>= 1;
		// read 1 wire
		*(lat+CLR) = mask; // drive port low
		*(tris+CLR) = mask; // set port to o/p
		*ifso = 0; *tmr = 0; *pmr = A;
		while(!(*ifso & ifsomask)); // wait for timer
		*(tris+SET) = mask; // release bus
		*ifso = 0; *tmr = 0; *pmr = E;
		while(!(*ifso & ifsomask)); // wait for timer
		inval = *port & mask;
		*ifso = 0; *tmr = 0; *pmr = F;
		while(!(*ifso & ifsomask)); // wait for timer
		if(inval) result |= 0x80;
	}
    return result;
}

// reads 1 bit
int owReadBit_pi(char *c)
{
int *ip, *tmr, *pmr, *ifso, ifsomask, *tris, *port, *lat, mask;
int loop, inval, result=0;
	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	tmr = (int*)*(ip);
	pmr = (int*)*(ip+1);
	ifso = (int*)*(ip+2);
	ifsomask = *(ip+3);
	tris = (int*)*(ip+4);
	port = (int*)*(ip+5);
	lat = (int*)*(ip+6);
	mask = *(ip+7);

	// read 1 wire
	*(lat+CLR) = mask; // drive port low
	*(tris+CLR) = mask; // set port to o/p
	*ifso = 0; *tmr = 0; *pmr = A;
	while(!(*ifso & ifsomask)); // wait for timer
	*(tris+SET) = mask; // release bus
	*ifso = 0; *tmr = 0; *pmr = E;
	while(!(*ifso & ifsomask)); // wait for timer
	result = *port & mask;
	*ifso = 0; *tmr = 0; *pmr = F;
	while(!(*ifso & ifsomask)); // wait for timer
    return result;
}

// writes one bit
int owWriteBit_pi(char *c)
{
int *ip, *tmr, *pmr, *ifso, ifsomask, *tris, *port, *lat, mask, value;
int loop, inval;
	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	tmr = (int*)*(ip);
	pmr = (int*)*(ip+1);
	ifso = (int*)*(ip+2);
	ifsomask = *(ip+3);
	tris = (int*)*(ip+4);
	port = (int*)*(ip+5);
	lat = (int*)*(ip+6);
	mask = *(ip+7);
	value = *(ip+10);

	// write 1 wire
	if(value & 1) { 
		// write '1' bit
		*(lat+CLR) = mask; // drive port low
		*(tris+CLR) = mask; // set port to o/p
		*ifso = 0; *tmr = 0; *pmr = A;
		while(!(*ifso & ifsomask)); // wait for timer
		*(tris+SET) = mask; // release bus
		*ifso = 0; *tmr = 0; *pmr = B;
		while(!(*ifso & ifsomask)); // wait for timer
	} else {
		// write '0' bit
		*(lat+CLR) = mask; // drive port low
		*(tris+CLR) = mask; // set port to o/p
		*ifso = 0; *tmr = 0; *pmr = C;
		while(!(*ifso & ifsomask)); // wait for timer
		*(tris+SET) = mask; // release bus
		*ifso = 0; *tmr = 0; *pmr = D;
		while(!(*ifso & ifsomask)); // wait for timer
	}
    return 0;
}

// see owRead_pi
int owWrite_pi(char *c)
{
int *ip, *tmr, *pmr, *ifso, ifsomask, *tris, *port, *lat, mask, value;
int loop, inval;
	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	tmr = (int*)*(ip);
	pmr = (int*)*(ip+1);
	ifso = (int*)*(ip+2);
	ifsomask = *(ip+3);
	tris = (int*)*(ip+4);
	port = (int*)*(ip+5);
	lat = (int*)*(ip+6);
	mask = *(ip+7);
	value = *(ip+10);

	for(loop=0; loop < 8; loop++) {
		// write 1 wire
		if(value & 1) { 
			// write '1' bit
			*(lat+CLR) = mask; // drive port low
			*(tris+CLR) = mask; // set port to o/p
			*ifso = 0; *tmr = 0; *pmr = A;
			while(!(*ifso & ifsomask)); // wait for timer
			*(tris+SET) = mask; // release bus
			*ifso = 0; *tmr = 0; *pmr = B;
			while(!(*ifso & ifsomask)); // wait for timer
		} else {
			// write '0' bit
			*(lat+CLR) = mask; // drive port low
			*(tris+CLR) = mask; // set port to o/p
			*ifso = 0; *tmr = 0; *pmr = C;
			while(!(*ifso & ifsomask)); // wait for timer
			*(tris+SET) = mask; // release bus
			*ifso = 0; *tmr = 0; *pmr = D;
			while(!(*ifso & ifsomask)); // wait for timer
		}
		value >>= 1;
	}
    return 0;
}

// see owRead_pi
int owReset_pi(char *c)
{
int *ip, *tmr, *pmr, *ifso, ifsomask, *tris, *port, *lat, mask, value;
int loop, inval, result=0;
	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	tmr = (int*)*(ip);
	pmr = (int*)*(ip+1);
	ifso = (int*)*(ip+2);
	ifsomask = *(ip+3);
	tris = (int*)*(ip+4);
	port = (int*)*(ip+5);
	lat = (int*)*(ip+6);
	mask = *(ip+7);
	value = *(ip+10);

	*tmr = 0; *pmr = G; *ifso = 0;
	while(!(*ifso & ifsomask)); // wait for timer
	*(lat+CLR) = mask; // drive port low
	*(tris+CLR) = mask; // set port to o/p
	*tmr = 0; *pmr = H; *ifso = 0;
	while(!(*ifso & ifsomask)); // wait for timer
	*(tris+SET) = mask; // release bus
	*tmr = 0; *pmr = I; *ifso = 0;
	while(!(*ifso & ifsomask)); // wait for timer
	result = *port & mask;
	*tmr = 0; *pmr = J; *ifso = 0;
	while(!(*ifso & ifsomask)); // wait for timer
    return result;
}

// should give 64uS
int test_pi(char *c)
{
int *ip, *tmr, *pmr, *ifso, ifsomask, *tris, *port, *lat, mask, value;
int loop, inval, result=0;
	// basic data as above.
	ip = (int*)c; // convenent integer pointer
	tmr = (int*)*(ip);
	pmr = (int*)*(ip+1);
	ifso = (int*)*(ip+2);
	ifsomask = *(ip+3);
	tris = (int*)*(ip+4);
	port = (int*)*(ip+5);
	lat = (int*)*(ip+6);
	mask = *(ip+7);
	value = *(ip+10);

	while(1) {
		*tmr = 0; *pmr = B; *ifso = 0;
		while(!(*ifso & ifsomask)); // wait for timer
		*(lat+CLR) = mask; // drive port low
		*(tris+CLR) = mask; // set port to o/p
		*tmr = 0; *pmr = B; *ifso = 0;
		while(!(*ifso & ifsomask)); // wait for timer
		*(tris+SET) = mask; // release bus
	}
}	
void main() {}

